unit DBEngn;

interface

uses
  Windows, Classes, SysUtils,
  MirGrobal, SerGrobal,TyFunc,
  M2Type, SDK;
type
  TDBEngine = class(TThread)

  private
    m_UserCriticalSection: TRTLCriticalSection;
    m_LoadRcdList: TList; //0x30
    m_SaveRcdList: TList; //0x34
    m_LoadRcdTempList: TList;
    m_SaveRcdTempList: TList;
    procedure GetGameTime();
    procedure ProcessGameDate();
    function LoadHumFromDB(LoadUser: pTLoadDBInfo; var boReTry: Boolean): Boolean;
    procedure Run();
    { Private declarations }
  protected
    procedure Execute; override;
  public
    constructor Create(CreateSuspended: Boolean);
    destructor Destroy; override;
    function SaveListCount(): Integer;
    function IsIdle(): Boolean;
    function IsFull(): Boolean;
    procedure DeleteHuman(nGateIndex, nSocket: Integer);
    function InSaveRcdList(sAccount, sChrName: string): Boolean;
    procedure AddToLoadRcdList(sAccount, sChrName, sIPaddr: string; boFlag: Boolean; nSessionID: Integer; nPayMent, nPayMode, nSoftVersionDate, nSocket, nGSocketIdx, nGateIdx: Integer);
    procedure AddToSaveRcdList(SaveRcd: pTSaveRcd);
  end;

implementation
uses
  M2Share, RunDB, ObjBase, ObjBActor;
{ TDBEngine }

constructor TDBEngine.Create(CreateSuspended: Boolean);
begin
  inherited;
  InitializeCriticalSection(m_UserCriticalSection);
  m_LoadRcdList := TList.Create;
  m_SaveRcdList := TList.Create;
  m_LoadRcdTempList := TList.Create;
  m_SaveRcdTempList := TList.Create;
end;

destructor TDBEngine.Destroy;
begin
  m_LoadRcdList.Free;
  m_SaveRcdList.Free;
  m_LoadRcdTempList.Free;
  m_SaveRcdTempList.Free;
  DeleteCriticalSection(m_UserCriticalSection);
  inherited;
end;
//004B5148

procedure TDBEngine.Execute;
const
  sExceptionMsg = '[쳣!!] TDBEngine::Execute';
begin
  while not Terminated do begin
    try
      Run();
    except
      MainOutMessage(sExceptionMsg);
    end;
    Sleep(1);
  end;
end;

procedure TDBEngine.GetGameTime; //004B50AC
var
  Hour, Min, Sec, MSec: Word;
begin
  DecodeTime(Time, Hour, Min, Sec, MSec);
  case Hour of
    5, 6, 7, 8, 9, 10, 16, 17, 18, 19, 20, 21, 22: g_nGameTime := 1;
    11, 23: g_nGameTime := 2;
    4, 15: g_nGameTime := 0;
    0, 1, 2, 3, 12, 13, 14: g_nGameTime := 3;
  end;
end;

function TDBEngine.IsIdle: Boolean;
begin
  Result := False;
  EnterCriticalSection(m_UserCriticalSection);
  try
    if m_SaveRcdList.Count = 0 then Result := True;
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;

function TDBEngine.SaveListCount: Integer;
begin
  Result := 0;
  EnterCriticalSection(m_UserCriticalSection);
  try
    Result := m_SaveRcdList.Count;
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;

procedure TDBEngine.ProcessGameDate;
var
  i: Integer;
  ii: Integer;
  TempList: TList;
  LoadDBInfo: pTLoadDBInfo;
  SaveRcd: pTSaveRcd;
  boReTryLoadDB: Boolean;
begin
  EnterCriticalSection(m_UserCriticalSection);
  try
    for i := 0 to m_SaveRcdList.Count - 1 do begin
      m_SaveRcdTempList.Add(m_SaveRcdList.Items[i]);
    end;

    TempList := m_LoadRcdTempList;
    m_LoadRcdTempList := m_LoadRcdList;
    m_LoadRcdList := TempList;

  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
  if DBSocketConnected then begin
    for i := 0 to m_SaveRcdTempList.Count - 1 do begin
      SaveRcd := m_SaveRcdTempList.Items[i];
      if SaveRcd = nil then Continue;
      if RunDB_SaveHumRcdToDB(SaveRcd) or (SaveRcd.nReTryCount > 50) then begin
        //MainOutMessage('SaveHumRcdToDB =true');
        if SaveRcd.PlayObject <> nil then begin
          TPlayObject(SaveRcd.PlayObject).m_boRcdSaved := True;
        end;
        EnterCriticalSection(m_UserCriticalSection);
        try
          for ii := 0 to m_SaveRcdList.Count - 1 do begin
            if m_SaveRcdList.Items[ii] = SaveRcd then begin
              m_SaveRcdList.Delete(ii);
              DisPose(SaveRcd);
              break;
            end;
          end;
        finally
          LeaveCriticalSection(m_UserCriticalSection);
        end;
      end else begin
        //޸߹һﲻ˳
        //MainOutMessage('SaveHumRcdToDB =FALSE');
        if g_boExitServer then begin
          if SaveRcd.PlayObject <> nil then begin
            TPlayObject(SaveRcd.PlayObject).m_boRcdSaved := True;
          end;
          EnterCriticalSection(m_UserCriticalSection);
          try
            for ii := 0 to m_SaveRcdList.Count - 1 do begin
              if m_SaveRcdList.Items[ii] = SaveRcd then begin
                m_SaveRcdList.Delete(ii);
                DisPose(SaveRcd);
                break;
              end;
            end;
          finally
            LeaveCriticalSection(m_UserCriticalSection);
          end;
        end;
        Inc(SaveRcd.nReTryCount);
      end;
    end;
  end else begin //DBѾرգڱ
    //MainOutMessage('not DBSocketConnected');
    EnterCriticalSection(m_UserCriticalSection);
    try
      for i := 0 to m_SaveRcdList.Count - 1 do begin
        if m_SaveRcdList.Items[i] <> nil then
          DisPose(m_SaveRcdList.Items[i]);
      end;
    finally
      LeaveCriticalSection(m_UserCriticalSection);
    end;
    m_SaveRcdList.Clear;
  end;
  m_SaveRcdTempList.Clear;

  for i := 0 to m_LoadRcdTempList.Count - 1 do begin
    LoadDBInfo := m_LoadRcdTempList.Items[i];
    if LoadDBInfo = nil then Continue;
    if not LoadHumFromDB(LoadDBInfo, boReTryLoadDB) then begin // ȡʧ
      RunSocket.CloseUser(LoadDBInfo.nGateIdx, LoadDBInfo.nSocket); //ر
    end else begin //ȡɹ
      if not boReTryLoadDB then begin //Ҫ?
        DisPose(LoadDBInfo);
      end else begin //ȡʧ(ݻûб),¼
        EnterCriticalSection(m_UserCriticalSection);
        try
          m_LoadRcdList.Add(LoadDBInfo);
        finally
          LeaveCriticalSection(m_UserCriticalSection);
        end;
      end;
    end;
  end;
  m_LoadRcdTempList.Clear;
end;

function TDBEngine.IsFull: Boolean; //004B4988
begin
  Result := False;
  EnterCriticalSection(m_UserCriticalSection);
  try
    if m_SaveRcdList.Count >= 1000 then begin
      Result := True;
    end;
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;

procedure TDBEngine.AddToLoadRcdList(sAccount, sChrName, sIPaddr: string;
  boFlag: Boolean; nSessionID, nPayMent, nPayMode, nSoftVersionDate, nSocket, nGSocketIdx, nGateIdx: Integer);
//004B46A0
var
  LoadRcdInfo: pTLoadDBInfo;
begin
  New(LoadRcdInfo);
  LoadRcdInfo.sAccount := sAccount;
  LoadRcdInfo.sCharName := sChrName;
  LoadRcdInfo.sIPaddr := sIPaddr;
  LoadRcdInfo.nSessionID := nSessionID;
  LoadRcdInfo.nClientFileCRC := nSoftVersionDate;
  LoadRcdInfo.nPayMent := nPayMent;
  LoadRcdInfo.nPayMode := nPayMode;
  LoadRcdInfo.nSocket := nSocket;
  LoadRcdInfo.nGSocketIdx := nGSocketIdx;
  LoadRcdInfo.nGateIdx := nGateIdx;
  LoadRcdInfo.dwNewUserTick := GetTickCount();
  LoadRcdInfo.PlayObject := nil;
  LoadRcdInfo.nReLoadCount := 0;

  EnterCriticalSection(m_UserCriticalSection);
  try
    m_LoadRcdList.Add(LoadRcdInfo);
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;

function TDBEngine.LoadHumFromDB(LoadUser: pTLoadDBInfo; var boReTry: Boolean): Boolean; //004B4B10
var
  HumanRcd: THumDataInfo;
  UserOpenInfo: pTUserOpenInfo;
  PlayA: TPlayObject;
  PlayB: TBActor;
begin
  PlayA := nil;
  PlayB := nil;
  Result := False;
  boReTry := False;
  if InSaveRcdList(LoadUser.sAccount, LoadUser.sCharName) then begin
    boReTry := True; //TRUE,¼
    Exit;
  end;
  if (UserEngine.GetPlayObjectEx(LoadUser.sAccount, LoadUser.sCharName) <> nil) then begin
    UserEngine.KickPlayObjectEx(LoadUser.sAccount, LoadUser.sCharName);
    boReTry := True; //TRUE,¼
    Exit;
  end;
  if not RunDB_LoadHumRcdFromDB(LoadUser.sAccount, LoadUser.sCharName, LoadUser.sIPaddr, HumanRcd, LoadUser.nSessionID) then begin
    MainOutMessage('ȡʧŶ :sCharName'+LoadUser.sCharName);
    RunSocket.SendOutConnectMsg(LoadUser.nGateIdx, LoadUser.nSocket, LoadUser.nGSocketIdx);
  end else begin
    if IsBitOn(HumanRcd.Data.bt8Boolean, HDBIIsHero) then begin
      //MainOutMessage('boIsHero Not AddUserOpenInfo:'+HumanRcd.Header.sName);
      PlayA := UserEngine.GetPlayObjectExOfAuto(LoadUser.sAccount);
      if PlayA <> nil then begin
        //MainOutMessage('PlayA :'+PlayA.m_sCharName);
        PlayB := TBActor.Create;
        UserEngine.GetHumData(PlayB, HumanRcd);
        UserEngine.MakePlayB(PlayA, PlayB);
      end;
      Result := True;
      Exit;
    end;
    New(UserOpenInfo);
    UserOpenInfo.sAccount := LoadUser.sAccount;
    UserOpenInfo.sChrName := LoadUser.sCharName;
    UserOpenInfo.LoadUser := LoadUser^;
    UserOpenInfo.HumanRcd := HumanRcd;
    UserEngine.AddUserOpenInfo(UserOpenInfo);
    //MainOutMessage('AddUserOpenInfo:'+HumanRcd.Header.sName);
    Result := True;
  end;
end;

function TDBEngine.InSaveRcdList(sAccount, sChrName: string): Boolean;
var
  i: Integer;
begin
  Result := False;
  EnterCriticalSection(m_UserCriticalSection);
  try
    for i := 0 to m_SaveRcdList.Count - 1 do begin
      if (pTSaveRcd(m_SaveRcdList.Items[i]).sAccount = sAccount) and
        (pTSaveRcd(m_SaveRcdList.Items[i]).sChrName = sChrName) then begin
        Result := True;
        break;
      end;
    end;
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;


procedure TDBEngine.AddToSaveRcdList(SaveRcd: pTSaveRcd); //004B49EC
begin
  EnterCriticalSection(m_UserCriticalSection);
  try
    m_SaveRcdList.Add(SaveRcd);
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;

procedure TDBEngine.DeleteHuman(nGateIndex, nSocket: Integer); //004B45EC
var
  i: Integer;
  LoadRcdInfo: pTLoadDBInfo;
begin
  EnterCriticalSection(m_UserCriticalSection);
  try
    for i := m_LoadRcdList.Count - 1 downto 0 do begin
      if m_LoadRcdList.Count <= 0 then break;
      LoadRcdInfo := m_LoadRcdList.Items[i];
      if LoadRcdInfo = nil then Continue;
      if (LoadRcdInfo.nGateIdx = nGateIndex) and (LoadRcdInfo.nSocket = nSocket) then begin
        DisPose(LoadRcdInfo);
        m_LoadRcdList.Delete(i);
        break;
      end;
    end;
  finally
    LeaveCriticalSection(m_UserCriticalSection);
  end;
end;



procedure TDBEngine.Run;
begin
  ProcessGameDate();
  GetGameTime();
end;

end.
