unit MapUnit;

interface

uses
  Windows, Classes, SysUtils,
  ClVarConst,
  ClType,
  TyFunc;

const
  MAPDIR = '.\Map\';
  MAXX = 40;
  MAXY = 40;

type




  TMapPrjInfo = packed record
    Ident: string[15];
    ColCount: integer;
    RowCount: integer;
  end;

  TMapHeader = packed record
    Width: word;
    Height: word;
    Title: string[15];
    UpdateDate: TDateTime;
    Reserved: array[0..23] of char;
  end;


  TMapInfo = packed record
    BkImg: word;
    MidImg: word;
    FrImg: word;
    DoorIndex: byte;
    DoorOffset: byte;
    AniFrame: byte;
    AniTick: byte;
    Area: byte;
    light: byte;
  end;
  PTMapInfo = ^TMapInfo;

  TMapInfoArr = array[0..MaxListSize] of TMapInfo;
  PTMapInfoArr = ^TMapInfoArr;

  TMap = class
  private
    procedure updatemap(cx, cy: integer);
  public
    MArr: array[0..MAXX * 3, 0..MAXY * 3] of TMapInfo;
    ClientRect: TRect;
    OldClientRect: TRect;
    BlockLeft, BlockTop: integer;
    oldleft, oldtop: integer;
    oldmap: string;
    CurUnitX, CurUnitY: integer;
    CurrentMap: string;
    FileName:  string;
    Mapwidth, MapHeight: Word;
    constructor Create;
    destructor Destroy;
    procedure UpdateMapSquare(cx, cy: integer);
    procedure UpdateMapPos(mx, my: integer);
    procedure ReadyReload;
    procedure LoadMap(mapname: string; mx, my: integer);
    procedure MarkCanWalk(mx, my: integer; bowalk: Boolean);
    function FixCanMove(mx, my: integer): Boolean;
    function CanFly(mx, my: integer): Boolean;
    function GetDoor(mx, my: integer): Integer;
    function IsDoorOpen(mx, my: integer): Boolean;
    function OpenDoor(mx, my: integer): Boolean;
    function CloseDoor(mx, my: integer): Boolean;
  end;

implementation

uses
  ClMain;


constructor TMap.Create;
begin
  inherited Create;

  ClientRect := Rect(0, 0, 0, 0);
  CurrentMap := '';
  CurUnitX := -1;
  CurUnitY := -1;
  BlockLeft := -1;
  BlockTop := -1;
  oldmap := '';
end;

destructor TMap.Destroy;
begin
  inherited Destroy;
end;


procedure TMap.updatemap(cx, cy: integer);
var
  fhandle, Offset,i, k, aline, lx, rx, ty, by: integer;
  header: TMapHeader;
begin
  FillChar(MArr, sizeof(MArr), 0);
  if CurrentMap ='3' then
    FileName := '.\Lcjdata\' + CurrentMap
  else
  FileName := MAPDIR + CurrentMap + '.map';
  if FileExists(FileName) then begin
    fhandle := FileOpen(FileName, fmOpenRead or fmShareDenyNone);
    if fhandle > 0 then begin
      FileRead(fhandle, header, sizeof(TMapHeader));
        Offset := sizeof(TMapHeader);

      lx := (cx - 1) * LOGICALMAPUNIT;
      rx := (cx + 2) * LOGICALMAPUNIT;
      ty := (cy - 1) * LOGICALMAPUNIT;
      by := (cy + 2) * LOGICALMAPUNIT;
      if lx < 0 then lx := 0;
      if ty < 0 then ty := 0;
      if by >= header.Height then by := header.Height;
      aline := sizeof(TMapInfo) * header.Height;
      MapWidth := Header.Width;
      MapHeight := Header.Height;
      for i := lx to rx - 1 do begin
        if (i >= 0) and (i < header.Width) then begin
          FileSeek(fhandle, Offset + (aline * i) + (sizeof(TMapInfo) * ty), 0);
          FileRead(fhandle, MArr[i - lx, 0], sizeof(TMapInfo) * (by - ty));
        end;
      end;
      FileClose(fhandle);
    end;
  end;
end;

procedure TMap.ReadyReload;
begin
  CurUnitX := -1;
  CurUnitY := -1;
end;


procedure TMap.UpdateMapSquare(cx, cy: integer);
begin
  if (cx <> CurUnitX) or (cy <> CurUnitY) then begin
    updatemap(cx, cy);
    CurUnitX := cx;
    CurUnitY := cy;
  end;
end;


procedure TMap.UpdateMapPos(mx, my: integer);
var
  cx, cy: integer;
begin
  cx := mx div LOGICALMAPUNIT;
  cy := my div LOGICALMAPUNIT;
  BlockLeft := _MAX(0, (cx - 1) * LOGICALMAPUNIT);
  BlockTop := _MAX(0, (cy - 1) * LOGICALMAPUNIT);

  UpdateMapSquare(cx, cy);

  oldleft := BlockLeft;
  oldtop := BlockTop;
end;


procedure TMap.LoadMap(mapname: string; mx, my: integer);
begin
  CurUnitX := -1;
  CurUnitY := -1;
  CurrentMap := mapname;
  UpdateMapPos(mx, my);
  oldmap := CurrentMap;
end;

procedure TMap.MarkCanWalk(mx, my: integer; bowalk: Boolean);
var
  cx, cy: integer;
begin
  cx := mx - BlockLeft;
  cy := my - BlockTop;
  if (cx < 0) or (cy < 0) then exit;
  if bowalk then
    Map.MArr[cx, cy].FrImg := Map.MArr[cx, cy].FrImg and $7FFF
  else
    Map.MArr[cx, cy].FrImg := Map.MArr[cx, cy].FrImg or $8000;
end;

function TMap.FixCanMove(mx, my: integer): Boolean;
var
  cx, cy: integer;
begin
  Result := False;
  cx := mx - BlockLeft;
  cy := my - BlockTop;
    if (cx > MAXX * 3) or (cy > MAXX * 3) then exit;
  if (cx < 0) or (cy < 0) then exit;
  Result := ((Map.MArr[cx, cy].BkImg and $8000) + (Map.MArr[cx, cy].FrImg and $8000)) = 0;
  if Result then begin  
    if Map.MArr[cx, cy].DoorIndex and $80 > 0 then begin
      if (Map.MArr[cx, cy].DoorOffset and $80) = 0 then
        Result := FALSE;
    end;
  end;
end;

function TMap.CanFly(mx, my: integer): Boolean;
var
  cx, cy: integer;
begin
  cx := mx - BlockLeft;
  cy := my - BlockTop;
  if (cx < 0) or (cy < 0) then exit;
  Result := (Map.MArr[cx, cy].FrImg and $8000) = 0;
  if Result then begin
    if Map.MArr[cx, cy].DoorIndex and $80 > 0 then begin
      if (Map.MArr[cx, cy].DoorOffset and $80) = 0 then
        Result := FALSE;
    end;
  end;
end;

function TMap.GetDoor(mx, my: integer): Integer;
var
  cx, cy: integer;
begin
  Result := 0;
  cx := mx - BlockLeft;
  cy := my - BlockTop;
  if Map.MArr[cx, cy].DoorIndex and $80 > 0 then begin
    Result := Map.MArr[cx, cy].DoorIndex and $7F;
  end;
end;

function TMap.IsDoorOpen(mx, my: integer): Boolean;
var
  cx, cy: integer;
begin
  Result := FALSE;
  cx := mx - BlockLeft;
  cy := my - BlockTop;
  if Map.MArr[cx, cy].DoorIndex and $80 > 0 then begin
    Result := (Map.MArr[cx, cy].DoorOffset and $80 <> 0);
  end;
end;

function TMap.OpenDoor(mx, my: integer): Boolean;
var
  i, j, cx, cy, idx: integer;
begin
  Result := FALSE;
  cx := mx - BlockLeft;
  cy := my - BlockTop;
  if (cx < 0) or (cy < 0) then exit;
  if Map.MArr[cx, cy].DoorIndex and $80 > 0 then begin
    idx := Map.MArr[cx, cy].DoorIndex and $7F;
    for i := cx - 10 to cx + 10 do
      for j := cy - 10 to cy + 10 do begin
        if (i > 0) and (j > 0) then
          if (Map.MArr[i, j].DoorIndex and $7F) = idx then
            Map.MArr[i, j].DoorOffset := Map.MArr[i, j].DoorOffset or $80;
      end;
  end;
end;

function TMap.CloseDoor(mx, my: integer): Boolean;
var
  i, j, cx, cy, idx: integer;
begin
  Result := FALSE;
  cx := mx - BlockLeft;
  cy := my - BlockTop;
  if (cx < 0) or (cy < 0) then exit;
  if Map.MArr[cx, cy].DoorIndex and $80 > 0 then begin
    idx := Map.MArr[cx, cy].DoorIndex and $7F;
    for i := cx - 8 to cx + 10 do
      for j := cy - 8 to cy + 10 do begin
        if (Map.MArr[i, j].DoorIndex and $7F) = idx then
          Map.MArr[i, j].DoorOffset := Map.MArr[i, j].DoorOffset and $7F;
      end;
  end;
end;





end.
