unit mywil;

interface

uses
  Windows, Classes, Graphics, SysUtils, Dialogs, DIB, DXDraws, DirectX;
const
  WithRegCom =True ;//
type
  TWMImageHeader = record
      Title: string[40];        //'WEMADE Entertainment inc.'
      ImageCount: integer;
      ColorCount: integer;
      PaletteSize: integer;
   end;
   TWMIndexHeader = record
      Title: string[40];        //'WEMADE Entertainment inc.'
      IndexCount: integer;
   end;
  TWMImageInfo = record
    Width: smallint;
    Height: smallint;
    px: smallint;
    py: smallint;
  end;
  TDXImage = record
    nPx: SmallInt;
    nPy: SmallInt;
    Surface: TDirectDrawSurface;
    dwNextTime: LongWord;
  end;
  pTDxImage = ^TDXImage;
  TDxImageArr = array[0..MaxListSize div 4] of TDxImage;
  PTDxImageArr = ^TDxImageArr;
  TTilFile = class{(TComponent)}
  private
    m_ImgArr: pTDxImageArr;
    FFileName: string;
    idxfile: string;
    FImageCount: Word;
    m_dwMemChecktTick: LongWord;
    Ftype:Byte;
    function LoadIndex(idxfile: string): Word;
    function GetCachedSurface(index: integer): TDirectDrawSurface;
    procedure LoadDxImage(position: integer; pdximg: PTDxImage);
  public
    lsDib: TDib;
    IndexList: array of Integer;
    Stream: TFileStream;
    IgCount: Integer;
    constructor Create(FileName: string);
    destructor Destroy; override;
    procedure Finalize;
    procedure FreeOldMemorys;
    function GetDxImage(index: integer): PTDxImage;
    function GetImage(index: integer; var px, py: SmallInt): TDirectDrawSurface;
    property Images[Index: Integer]: TDirectDrawSurface read GetCachedSurface; default;
  published
    property ImageCount: Word read FImageCount;
  end;

var
  MainPalette: TRGBQuads;
  boReadMainPalette: Boolean;
implementation
{$IF WithRegCom}  
{$ELSE}  
  uses ClMain, ResManager,Debug;  
{$IFEND}  

constructor TTilFile.Create(FileName: string);
var
  Header:TWMImageHeader;
begin
  inherited Create{(nil)};
  FFileName := '';
  FImageCount := 0;
  Stream := nil;
  m_ImgArr := nil;
  lsDib := TDib.Create;
  FFileName := FileName;
  m_dwMemChecktTick := GetTickCount;
  if (FileExists(FFileName+'.wil'))or(FileExists(FFileName+'.Til')) then begin
    //DebugOutStr('TTilFile.Create:: ļ'+FileName);
    if (FileExists(FFileName+'.wil')) then
      Stream := TFileStream.Create(FFileName+'.wil', fmOpenRead or fmShareDenyNone) else
    if (FileExists(FFileName+'.Til')) then
      Stream := TFileStream.Create(FFileName+'.til', fmOpenRead or fmShareDenyNone);
    if Stream <> nil then
    begin
      Stream.Read(Header,SizeOf(TWMImageHeader));
      if (Header.ColorCount = 256) then begin
        Stream.Read (MainPalette, sizeof(TRgbQuad) * 256);
        Ftype :=1
      end
      else if  (Header.ColorCount = 65536) then Ftype :=2
      else Ftype :=3;

      if Header.Title='WEMADE Entertainment inc.' then
        FType:=0;
      if Ftype <=1 then begin
        lsDib.BitCount := 8;
      end else begin
        with lsDib.PixelFormat do begin
          RBitMask := $F800;
          GBitMask := $07E0;
          BBitMask := $001F;
        end;
        lsDib.BitCount := 16;
      end;


      idxfile := FFileName + '.wix';
      FImageCount := LoadIndex(idxfile);
      GetMem(m_ImgArr,SizeOf(TDxImage) * FImageCount);
      FillChar(m_ImgArr^,SizeOf(TDxImage) * FImageCount,0);
      if m_ImgArr = nil then
        raise Exception.Create(' ImgArr = nil');
    end
    else
      MessageDlg(FFileName + ' Tilļ.', mtWarning, [mbOk], 0);
  end else Exit;
end;

destructor TTilFile.Destroy;
begin
  Finalize;
  if Stream <> nil then Stream.Free;
  lsDib.Free;
  FreeMem(m_ImgArr);
  inherited Destroy;
end;

procedure TTilFile.FreeOldMemorys;
var
  i,Code: integer;
begin
  try
    Code := 0;
    for i := 0 to ImageCount - 1 do begin
      Code := 1;
      if m_ImgArr[i].Surface <> nil then begin
        Code := 2;
        if GetTickCount - m_ImgArr[i].dwNextTime > 5 * 60 * 1000 then begin
          Code := 3;
          m_ImgArr[i].Surface.Free;
          Code := 4;
          m_ImgArr[i].Surface := nil;
        end;
      end;
    end;
  except
    {$IF WithRegCom}
    {$ELSE}
      DebugOutStr(FFileName + '[Exception] LoadDxImage 2 - ' + IntToStr(i)+':'+IntToStr(Code));
    {$IFEND}
  end;
end;



procedure TTilFile.Finalize;
var
  i: integer;
begin
  for i := 0 to FImageCount - 1 do begin
    if m_ImgArr[i].Surface <> nil then begin
      m_ImgArr[i].Surface.Free;
      m_ImgArr[i].Surface := nil;
    end;
  end;
  if Stream <> nil then
  begin
    Stream.Free;
    Stream := nil;
  end;
  SetLength(IndexList, 0);
  FImageCount := 0;
end;

function TTilFile.LoadIndex(idxfile: string): Word;
var
  FileStream: TFileStream;
  IndexHeader:TWMIndexHeader;
begin
  Result := 0;
  if FileExists(idxfile) then
  begin
    FileStream := TFileStream.Create(idxfile,fmOpenRead or fmShareDenyNone);
    if FileStream <> nil then
    begin
      //if FType = 0 then
      //ShowMessage(idxfile);
      if Ftype = 3 then begin
        SetLength(IndexList, (FileStream.Size div 4));
        Result := Length(IndexList);
        FileStream.Read(IndexList[0],Result*4);
      end else begin
        FileStream.Read(IndexHeader,SizeOf(TWMIndexHeader));
        SetLength(IndexList, IndexHeader.IndexCount);
        Result := Length(IndexList);
        FileStream.Seek(48, 0);
        FileStream.Read(IndexList[0],Result*4);
      end;

      if Ftype = 0 then begin
        FileStream.Seek(52, 0);
        FileStream.Read(IndexList[0],Result*4);
        if IndexList[0] <> 1084 then begin
          FileStream.Seek(48, 0);
          FileStream.Read(IndexList[0],Result*4);
          Ftype := 1;
        end;
      end;

      Result := Length(IndexList);
      FileStream.Free;
    end;
  end;
end;


procedure TTilFile.LoadDxImage(position: integer; pdximg: PTDxImage);
var
  imginfo: TWMImageInfo;
  ddsd: TDDSurfaceDesc;
  SBits, PSrc, DBits: PByte;
  n, slen, dlen: integer;
  nErrorCode: Integer;
  NewSurface: TDirectDrawSurface;
begin
  pdximg.surface := nil;
  try
    nErrorCode := 0;
    Stream.Seek(position, 0);
    if FType <> 0 then
      Stream.Read(imginfo, SizeOf(TWMImageInfo))
    else Stream.Read(imginfo, SizeOf(TWMImageInfo)+4);
    if imginfo.Width + imginfo.Height < 8 then Exit;
    nErrorCode := 1;

      lsDib.SetSize(imginfo.Width, imginfo.Height, lsDib.BitCount);
      lsDib.Fill(0);

    nErrorCode := 2;
    DBits := lsDib.PBits;
    if lsDib.BitCount = 8 then begin
      lsDib.ColorTable := MainPalette;
      lsDib.UpdatePalette;
      Stream.Read (DBits^, imginfo.Width * imginfo.Height);
    end else
      Stream.Read(DBits^, (imginfo.Width * imgInfo.Height) * 2);
    nErrorCode := 3;
    //try
      pdximg.nPx := imginfo.px;
      pdximg.nPy := imginfo.py;
      {$IF WithRegCom}
        NewSurface     := TDirectDrawSurface.Create(nil);
      {$ELSE}
        NewSurface     := TDirectDrawSurface.Create(WILCSDD);
      {$IFEND}

      NewSurface.SystemMemory := TRUE;
      NewSurface.SetSize(imginfo.Width, imginfo.Height);
      nErrorCode := 4;
      NewSurface.Canvas.Draw(0, 0, lsDib);
      //pdximg.surface.Canvas.TextOut(0,0,Copy(Self.FFileName,Length(Self.FFileName)-4,5));
      NewSurface.Canvas.Release;

      NewSurface.TransparentColor := 0;
    {except
      on e: Exception do begin
      //DebugOutStr('[Exception] TWMImages.LoadDxImage 12');
      //DebugOutStr(E.Message);
      end;
    end;  }
  except
    {$IF WithRegCom}
    {$ELSE}
       DebugOutStr(FFileName + '[Exception] LoadDxImage 2 - ' + IntToStr(nErrorCode));
    {$IFEND}
  end;
  pdximg.Surface := NewSurface;
end;


function TTilFile.GetCachedSurface(index: integer): TDirectDrawSurface;
var
  nPosition: Integer;
  nErrCode: Integer;
begin
  Result := nil;
  nErrCode := 0;
  try
    if (ImageCount <> 0) and (index < 0) or (index > ImageCount) then exit;
    if GetTickCount - m_dwMemChecktTick > 10000 then begin
      m_dwMemChecktTick := GetTickCount;
      //if MemorySize > FMaxMemorySize then begin
      FreeOldMemorys;
      //end;
    end;
    nErrCode := 1;
    if m_ImgArr[index].Surface = nil then begin //cacheǾ  .  о.
      if index <= High(IndexList) then begin
        nPosition := IndexList[index];
        LoadDxImage(nPosition, @m_ImgArr[index]);
        m_ImgArr[index].dwNextTime := GetTickCount;// + 60 * 1000 * 5;
        nErrCode := 2;
        Result := m_ImgArr[index].Surface;
         //MemorySize := MemorySize + ImgArr[index].Surface.Width * ImgArr[index].Surface.Height;
      end;
    end else begin
      m_ImgArr[index].dwNextTime := GetTickCount;// + 60 * 1000 * 5;
      nErrCode := 3;
      Result := m_ImgArr[index].Surface;
    end;

  except
  {$IF WithRegCom}
  {$ELSE}
    DebugOutStr(FFileName + 'GetCachedSurface 3 Index: ' + IntToStr(index) + ' Error Code: ' + IntToStr(nErrCode));
  {$IFEND}
  end;
end;
function TTilFile.GetImage(index: integer; var px, py: SmallInt): TDirectDrawSurface;
begin
  if(ImageCount <> 0) and (index >= 0) and (index < ImageCount) then begin
    if m_ImgArr[index].surface = nil then GetCachedSurface(index);
    Result := m_ImgArr[index].surface;
    m_ImgArr[index].dwNextTime := GetTickCount;// + 60 * 1000 * 5;
    px := m_ImgArr[index].nPx;
    py := m_ImgArr[index].nPy;
  end else
    Result := nil;
end;
function TTilFile.GetDxImage(index: integer): PTDxImage;
begin
  if (ImageCount <> 0) and (index >= 0) and (index < ImageCount) then begin
    if m_ImgArr[index].surface = nil then GetCachedSurface(index);
    Result := @m_ImgArr[index];
    m_ImgArr[index].dwNextTime := GetTickCount;// + 60 * 1000 * 5;
  end else
    Result := nil;
end;
end.
